# sb::Plugin::ReplaceText - Plugin for Serene Bach
# == written by T.Otani <takuya.otani@gmail.com> ===
# == Copyright (C) 2004 SimpleBoxes/SerendipityNZ ==

# 0.01 [2012/12/15] created

package sb::Plugin::ReplaceText;
# ==================================================
# // initialization for plugin
# ==================================================
use sb::Plugin ();

sb::Plugin->register_plugin(
	'lang' => {
		'ja' => 'euc',
		'en' => 'ascii',
	},
	'text' => {
		'type'    => 'admin',
		'name'    => 'Replace Text',
		'text'    => 'Find & replace text in entry body/sequel',
		'author'  => 'takkyun',
		'detail'  => 'http://serenebach.net/',
		'version' => version(),
	},
	'file' => 'replacetext.txt',
	'data' => undef,
);

sb::Plugin->register_admin_module(
	'mode'   => 'replacetext',
	'level'  => 1,
	'module' => 'sb::Admin::ReplaceText',
);
sub version
{
	return '0.01';
	# 0.01 [2012/12/15] created
}
# ======================================
package sb::Admin::ReplaceText;
use strict;
use vars qw( @ISA );
use sb::Text ();
use sb::Data ();
use sb::Admin::List ();
@ISA = qw( sb::Admin::List );
# --------------------------------------
# // defines
sub TEMPLATE (){ 'replacetext.html' };
# --------------------------------------
# // public functions
sub callback
{
	my $self = shift;
	return ( $self->{'regi'} ) 
		? $self->_do_search(@_)
		: $self->_open_form(@_);
}
# --------------------------------------
# // private functions
sub _do_search
{
	my $self = shift;
	my %param = ( 'message' => '', @_ );
	my $msg = '';
	my $cgi  = sb::Interface->get;
	my $lang = sb::Language->get;
	$self->_setup_target_entries($cgi->value('search_text'));
	my $num = @{$self->{'entries'}};
	if ($num > 0)
	{
		$msg .= $num;
		if ($cgi->value('replace_check') eq 'on')
		{ # do replace
			$self->_do_replace($cgi->value('search_text'),$cgi->value('replace_text'));
			$msg .= $lang->string('parts_replaced');
			$msg .= '<br />' . $lang->string('parts_rec_make');
			$msg .= sprintf($lang->string('parts_link_bld'),$self->get_script_path);
		}
		else
		{
			$msg .= $lang->string('parts_found');
		}
	}
	else
	{
		$msg .= $lang->string('parts_not_found');
	}
	return $self->_open_form('message'=>$msg);
}
sub _open_form
{
	my $self = shift;
	my %param = ( 'message' => '', @_ );
	my $cgi = sb::Interface->get;
	my $cms = sb::TemplateManager->new(sb::Plugin->load_template('file'=>TEMPLATE));
	if ($param{'message'} ne '')
	{
		$cms->num(0);
		$cms->tag('sb_process_message'=>$param{'message'});
		$cms->block('sb_process_message'=>1);
	}
	if ($self->{'entries'} and @{$self->{'entries'}} > 0)
	{
		$self->_display_results($cms);
	}
	$cms->num(0);
	$cms->block('sb_replacetext_form' => 1);
	$cms->tag('sb_search_text'=>sb::Text->entitize($cgi->value('search_text'))) if ($cgi->value('search_text') ne '');
	if ($cgi->value('replace_check') eq 'on')
	{
		$cms->tag('sb_replace_text'=>sb::Text->entitize($cgi->value('replace_text'))) if ($cgi->value('replace_text') ne '');
		$cms->tag('sb_replace_checked'=>'checked="checked"');
	}
	$self->common_template_parts($cms);
	return sb::Interface->get->head('type'=>'text/html') . $self->set_main($cms->output);
}
sub _setup_target_entries
{
	my $self = shift;
	my $word = shift;
	if ( ! defined $self->{'cat'} )
	{
		$self->{'cat'} = { sb::Data->load_as_hash('Category') };
	}
	if ( $word ne '' and ! defined $self->{'entries'} )
	{
		$self->{'entries'} = [
			sb::Data->load('Entry',
				'cond'   => {
					'body' => '/' . $word . '/',
					'more' => '/' . $word . '/',
				},
				'logic'  => 'or',  # logic operator for condition
				'bgn'    => 0,     # offset for range
				'num'    => -1,    # number of range
				'sort'   => 'id',  # sorting option
				'order'  => 1,     # order [descent/ascent]
				'detail' => 1,     # if set true, loading objects with details
			)
		];
	}
	else
	{
		$self->{'entries'} = [];
	}
}
sub _do_replace
{
	my $self = shift;
	my $orig = shift;
	my $dest = shift;
	foreach my $entry (@{$self->{'entries'}})
	{
		my $body = $entry->body;
		my $more = $entry->more;
		$body =~ s/\Q$orig\E/$dest/gi;
		$more =~ s/\Q$orig\E/$dest/gi;
		$entry->body($body);
		$entry->more($more);
	}
	sb::Data->update(@{$self->{'entries'}});
}
sub _display_results
{
	my $self = shift;
	my $cms = shift;
	$cms->num(0);
	$cms->block('sb_replacetext_result'=>1);
	$self->listmain(
		'template' => $cms,
		'block'    => 'sb_entry_list',
		'objects'  => $self->{'entries'},
		'tags'     => {
			'sb_entry_id'        => 'id',
			'sb_entry_title'     => \&sb::Admin::List::_clip_for_entry,
			'sb_entry_author'    => \&sb::Admin::List::_display_author,
			'sb_entry_category'  => \&sb::Admin::List::_display_category,
			'sb_entry_date'      => 'date',
			'sb_entry_status'    => \&sb::Admin::List::_display_entry_status,
			'sb_entry_catclass'  => 'cat',
			'sb_entry_statclass' => \&sb::Admin::List::_display_entry_statnum,
		}
	);
}
1;
__END__
